/* Project name:
     MP3
 * Copyright:
     (c) MikroElektronika, 2010.
 * Revision History:
     20101112 (TL):
       - initial release;
     20110906(JK):
       - modified for Hw. Rev. 1.10
 * Description:
     MP3 module test. Upon flashing, insert a MMC/SD card into the
     module, when you should receive the "Init-OK" message.
     Then, you can experiment with SD read and write functions,
     and observe the results through the Usart Terminal (Ctrl + T).
 * Test configuration:
     MCU:             PIC24FJ256GB110
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39897c.pdf
     Dev.Board:       mikroMMB for PIC24
                      http://www.mikroe.com/eng/products/view/587/mikrommb-for-pic24-board/
     Oscillator:      XT-PLL 4x, 08.00000 MHz
     Modules:         ac:MP3
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/eng/products/view/231/mikroc-pro-for-dspic30-33-and-pic24/
 * NOTES:
     - This is NOT an example for working with MP3 FILES on a MMC/SD memory module.
     - This example should work both for MMC and SD media types.
 */
 
 #include <built_in.h>

unsigned short TFT_DataPort at LATE;                  // DATA port
unsigned short TFT_DataPort_Direction at TRISE;       // DATA port

sbit TFT_RST at LATC1_bit;
sbit TFT_RS at LATB15_bit;
sbit TFT_CS at LATF12_bit;
sbit TFT_RD at LATD5_bit;
sbit TFT_WR at LATD4_bit;

sbit TFT_RS_Direction  at TRISB15_bit;
sbit TFT_RST_Direction at TRISC1_bit;
sbit TFT_CS_Direction  at TRISF12_bit;
sbit TFT_RD_Direction  at TRISD5_bit;
sbit TFT_WR_Direction  at TRISD4_bit;


// MP3 connections
sbit Mmc_Chip_Select           at LATG9_bit;
sbit MP3_CS                    at LATA7_bit;
sbit MP3_RST                   at LATA5_bit;
sbit DREQ                      at RA4_bit;
sbit BSYNC                     at LATG12_bit;

sbit Mmc_Chip_Select_Direction at TRISG9_bit;
sbit MP3_CS_Direction          at TRISA7_bit;
sbit MP3_RST_Direction         at TRISA5_bit;
sbit DREQ_Direction            at TRISA4_bit;
sbit BSYNC_Direction           at TRISG12_bit;

sbit TFT_BLED at LATD2_bit;
sbit TFT_BLED_Direction at TRISD2_bit;

const char WRITE_CODE  = 0x02;
const char READ_CODE   = 0x03;
const char SCI_MODE_ADDR   = 0x00;
const char SCI_CLOCKF_ADDR = 0x03;
const char SCI_VOL_ADDR    = 0x0B;
const char SCI_BASS_ADDR = 0x02;

// global variables
char          filename[14] = "sound1.mp3";   // MP3 file name
unsigned long i, file_size;
char  data_buffer_32[32];
const BUFFER_SIZE = 512;
char  BufferLarge[BUFFER_SIZE];
char  volume_left, volume_right;
unsigned int SCI_regs[20];
char sci;

// Write text and goto new line CR and LF
void UART2_Write_Line (char *text) {
  UART2_Write_Text(text);
  UART2_Write(13);
  UART2_Write(10);
}

// Writes one word to MP3 SCI
void MP3_SCI_Write(char address, unsigned int data_in) {
  BSYNC = 1;
  
  MP3_CS = 0;                             // select MP3 SCI
  SPI2_Write(WRITE_CODE);
  SPI2_Write(address);
  SPI2_Write(Hi(data_in));                // high byte
  SPI2_Write(Lo(data_in));                // low byte
  MP3_CS = 1;                             // deselect MP3 SCI
  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

// Reads words_count words from MP3 SCI
void MP3_SCI_Read(char start_address, char words_count, unsigned int *data_buffer) {
  unsigned int temp;

  BSYNC = 1;

  MP3_CS = 0;                             // select MP3 SCI
  SPI2_Write(READ_CODE);
  SPI2_Write(start_address);

  while (words_count--) {                 // read words_count words byte per byte
    temp = SPI2_Read(0);
    temp <<= 8;
    temp += SPI2_Read(0);
    *(data_buffer++) = temp;
    while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
  }
  MP3_CS = 1;                             // deselect MP3 SCI

}

// Write one byte to MP3 SDI
void MP3_SDI_Write(char data_) {

  MP3_CS = 1;
  BSYNC = 0;

  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  SPI2_Write(data_);
  BSYNC = 1;
}

// Write 32 bytes to MP3 SDI
void MP3_SDI_Write_32(char *data_) {
  char i;

  MP3_CS = 1;
  BSYNC = 0;

  while (DREQ == 0);                      // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  for (i=0; i<32; i++)
  SPI2_Write(data_[i]);
  BSYNC = 1;
}


// Set volume
void Set_Volume(char left,char right) {
  unsigned int volume;

  volume = (left<<8) + right;             // calculate value
  MP3_SCI_Write(SCI_VOL_ADDR, volume);        // Write value to VOL register
}

// Initialize VS1011E
void MP3_Init() {

  BSYNC = 1;
  MP3_CS = 1;

  // Hardware reset
  MP3_RST = 0;
  Delay_ms(10);
  MP3_RST = 1;

  while (DREQ == 0);

  MP3_SCI_Write(SCI_MODE_ADDR, 0x0800);
  MP3_SCI_Write(SCI_BASS_ADDR, 0x7A00);
  MP3_SCI_Write(SCI_CLOCKF_ADDR, 0x2000); // default 12 288 000 Hz


  volume_left  = 10;                      // Set volume to initial value
  volume_right = 10;
  Set_Volume(volume_left,volume_right);
}

void Init() {
  ADPCFG = 0xFFFF;        // set all AN pins to digital

  MP3_CS_Direction  = 0;                  // Configure MP3_CS as output
  MP3_CS            = 1;                  // Deselect MP3_CS
  MP3_RST_Direction = 0;                  // Configure MP3_RST as output
  MP3_RST           = 1;                  // Set MP3_RST pin

  DREQ_Direction  = 1;                    // Configure DREQ as input
  BSYNC_Direction = 0;                    // Configure BSYNC as output
  BSYNC           = 0;                    // Clear BSYNC
  BSYNC           = 1;                    // Clear BSYNC

  TFT_BLED_Direction = 0;
  TFT_BLED = 0;
}

void out(unsigned int val){
 char txt[5];
 WordToHex(val,txt);

}

void main() {

  //Disable other peripheral modules on the same SPI bus
  TRISC2_bit =0;
  LATC2_bit =1;  //Disable Serial Flash
  //
  

  PORTB = 0;
  PORTD = 0;
  AD1PCFGL = 0xFFFF;
  AD1PCFGH = 0xFFFF;

  PPS_Mapping(19, _OUTPUT, _SDO2);
  PPS_Mapping(21, _OUTPUT, _SCK2OUT);
  PPS_Mapping(26 ,_INPUT, _SDI2);

  PPS_Mapping(24, _OUTPUT, _U2TX);
  PPS_Mapping(22, _INPUT,  _U2RX);

  delay_ms(100);
  delay_ms(100);
  TFT_Init(320,240);
  TFT_Fill_Screen(CL_SILVER);
   Init();

  // Initialize USART for signaling
  UART2_Init(9600);
  Delay_100ms();

  UART2_Write_Line("Initializing SPI");
  SPI2_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, _SPI_PRESCALE_SEC_4, _SPI_PRESCALE_PRI_4,
                     _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_HIGH, _SPI_ACTIVE_2_IDLE);

  UART2_Write_Line("Initializing VS1011E decoder interface");
  MP3_init();

  UART2_Write_Line("Initializing MMC_FAT");
  if (Mmc_Fat_Init() == 0) {
    if (Mmc_Fat_Assign(&filename, 0) ) {
      UART2_Write_Line("File Assigned");
      Mmc_Fat_Reset(&file_size);          // Call Reset before file reading,
                                          //   procedure returns size of the file
      // send file blocks to MP3 SDI
      while (file_size > BUFFER_SIZE) {

        for (i=0; i<BUFFER_SIZE; i++) {
          Mmc_Fat_Read(BufferLarge + i);
        }

        for (i=0; i<BUFFER_SIZE/32; i++) {
          MP3_SDI_Write_32(BufferLarge + i*32);
        }

        file_size -= BUFFER_SIZE;

      }

      // send the rest of the file to MP3 SDI
      for (i=0; i<file_size; i++) {
        Mmc_Fat_Read(BufferLarge + i);
      }
      for (i=0; i<file_size; i++) {
        MP3_SDI_Write(BufferLarge[i]);
      }
    }
    else {
      UART2_Write_Line("File not assigned");
    }

  }
  else {
    UART2_Write_Line("MMC FAT not initialized");
  }
}